@@ -25,9 +25,9 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
-from retrying import retry
 import six
 from six.moves.urllib import parse as urlparse
+import tenacity
 
 from heat.common import exception
 from heat.common.i18n import _
@@ -40,7 +40,6 @@ from heat.engine import constraints
 LOG = logging.getLogger(__name__)
 
 
-NOVA_API_VERSION = "2.1"
 CLIENT_NAME = 'nova'
 
 
@@ -59,23 +58,44 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
 
     exceptions_module = exceptions
 
+    supported_versions = [
+        NOVA_API_VERSION, V2_2, V2_26
+    ] = [
+        '2.1', '2.2', '2.26'
+    ]
+
     service_types = [COMPUTE] = ['compute']
 
-    def _create(self):
+    def _get_service_name(self):
+        return self.COMPUTE
+
+    def _create(self, version=None):
+        if not version:
+            # TODO(prazumovsky): remove all unexpected calls from tests and
+            # add default_version after that.
+            version = self.NOVA_API_VERSION
         endpoint_type = self._get_client_option(CLIENT_NAME, 'endpoint_type')
-        extensions = nc.discover_extensions(NOVA_API_VERSION)
+        extensions = nc.discover_extensions(version)
 
         args = {
             'session': self.context.keystone_session,
             'extensions': extensions,
-            'interface': endpoint_type,
+            'endpoint_type': endpoint_type,
             'service_type': self.COMPUTE,
             'region_name': self._get_region_name(),
             'http_log_debug': self._get_client_option(CLIENT_NAME,
                                                       'http_log_debug')
         }
 
-        client = nc.Client(NOVA_API_VERSION, **args)
+        client = nc.Client(version, **args)
+        # NOTE: check for microversion availability
+        if version in [self.V2_2, self.V2_26]:
+            try:
+                client.versions.get_current()
+            except exceptions.NotAcceptable:
+                raise exception.InvalidServiceVersion(service=self.COMPUTE,
+                                                      version=version)
+
         return client
 
     def is_not_found(self, ex):
@@ -96,8 +116,12 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
         return (isinstance(ex, exceptions.ClientException) and
                 http_status == 422)
 
-    @retry(stop_max_attempt_number=max(cfg.CONF.client_retry_limit + 1, 0),
-           retry_on_exception=client_plugin.retry_if_connection_err)
+    @tenacity.retry(
+        stop=tenacity.stop_after_attempt(
+            max(cfg.CONF.client_retry_limit + 1, 0)),
+        retry=tenacity.retry_if_exception(
+            client_plugin.retry_if_connection_err),
+        reraise=True)
     def get_server(self, server):
         """Return fresh server object.
 
@@ -301,7 +325,6 @@ class NovaClientPlugin(client_plugin.ClientPlugin):
                 subtype = os.path.splitext(filename)[0]
             if content is None:
                 content = ''
-
             try:
                 content.encode('us-ascii')
                 charset = 'us-ascii'
@@ -343,7 +366,6 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                        (cloudinit_boothook, 'boothook.sh', 'cloud-boothook'),
                        (read_cloudinit_file('part_handler.py'),
                         'part-handler.py')]
-
         if is_cfntools:
             attachments.append((userdata, 'cfn-userdata', 'x-cfninitdata'))
         elif is_software_config:
@@ -360,7 +382,7 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                                         part.get_filename(),
                                         part.get_content_subtype()))
             else:
-                attachments.append((userdata, 'userdata', 'x-shellscript'))
+                attachments.append((userdata, ''))
 
         if is_cfntools:
             attachments.append((read_cloudinit_file('loguserdata.py'),
@@ -553,8 +575,12 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
                 if len(server.networks[n]) > 0:
                     return server.networks[n][0]
 
-    @retry(stop_max_attempt_number=max(cfg.CONF.client_retry_limit + 1, 0),
-           retry_on_exception=client_plugin.retry_if_connection_err)
+    @tenacity.retry(
+        stop=tenacity.stop_after_attempt(
+            max(cfg.CONF.client_retry_limit + 1, 0)),
+        retry=tenacity.retry_if_exception(
+            client_plugin.retry_if_connection_err),
+        reraise=True)
     def absolute_limits(self):
         """Return the absolute limits as a dictionary."""
         limits = self.client().limits.get()
@@ -670,12 +696,11 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
             return False
 
     def interface_detach(self, server_id, port_id):
-        server = self.fetch_server(server_id)
-        if server:
-            server.interface_detach(port_id)
-            return True
-        else:
-            return False
+        with self.ignore_not_found:
+            server = self.fetch_server(server_id)
+            if server:
+                server.interface_detach(port_id)
+                return True
 
     def interface_attach(self, server_id, port_id=None, net_id=None, fip=None):
         server = self.fetch_server(server_id)
@@ -685,21 +710,26 @@ echo -e '%s\tALL=(ALL)\tNOPASSWD: ALL' >> /etc/sudoers
         else:
             return False
 
-    @retry(stop_max_attempt_number=cfg.CONF.max_interface_check_attempts,
-           wait_fixed=500,
-           retry_on_result=client_plugin.retry_if_result_is_false)
+    @tenacity.retry(
+        stop=tenacity.stop_after_attempt(
+            cfg.CONF.max_interface_check_attempts),
+        wait=tenacity.wait_fixed(0.5),
+        retry=tenacity.retry_if_result(client_plugin.retry_if_result_is_false))
     def check_interface_detach(self, server_id, port_id):
-        server = self.fetch_server(server_id)
-        if server:
-            interfaces = server.interface_list()
-            for iface in interfaces:
-                if iface.port_id == port_id:
-                    return False
+        with self.ignore_not_found:
+            server = self.fetch_server(server_id)
+            if server:
+                interfaces = server.interface_list()
+                for iface in interfaces:
+                    if iface.port_id == port_id:
+                        return False
         return True
 
-    @retry(stop_max_attempt_number=cfg.CONF.max_interface_check_attempts,
-           wait_fixed=500,
-           retry_on_result=client_plugin.retry_if_result_is_false)
+    @tenacity.retry(
+        stop=tenacity.stop_after_attempt(
+            cfg.CONF.max_interface_check_attempts),
+        wait=tenacity.wait_fixed(0.5),
+        retry=tenacity.retry_if_result(client_plugin.retry_if_result_is_false))
     def check_interface_attach(self, server_id, port_id):
         server = self.fetch_server(server_id)
         if server:
