@@ -29,10 +29,15 @@ class DesignateRecord(resource.Resource):
     """
 
     support_status = support.SupportStatus(
-        version='5.0.0')
+        status=support.DEPRECATED,
+        version='8.0.0',
+        message=_('Use OS::Designate::RecordSet instead.'),
+        previous_status=support.SupportStatus(version='5.0.0'))
 
     entity = 'records'
 
+    default_client_name = 'designate'
+
     PROPERTIES = (
         NAME, TTL, DESCRIPTION, TYPE, DATA, PRIORITY, DOMAIN
     ) = (
@@ -104,8 +109,6 @@ class DesignateRecord(resource.Resource):
         ),
     }
 
-    default_client_name = 'designate'
-
     def handle_create(self):
         args = dict(
             name=self.properties[self.NAME],
