@@ -78,7 +78,7 @@ class ManilaShare(resource.Resource):
             _('Share protocol supported by shared filesystem.'),
             required=True,
             constraints=[constraints.AllowedValues(
-                ['NFS', 'CIFS', 'GlusterFS', 'HDFS'])]
+                ['NFS', 'CIFS', 'GlusterFS', 'HDFS', 'CEPHFS'])]
         ),
         SIZE: properties.Schema(
             properties.Schema.INTEGER,
@@ -139,7 +139,7 @@ class ManilaShare(resource.Resource):
                         properties.Schema.STRING,
                         _('Type of access that should be provided to guest.'),
                         constraints=[constraints.AllowedValues(
-                            ['ip', 'domain'])],
+                            ['ip', 'user', 'cert', 'cephx'])],
                         required=True
                     ),
                     ACCESS_LEVEL: properties.Schema(
@@ -195,6 +195,8 @@ class ManilaShare(resource.Resource):
         return self.client().shares.get(self.resource_id)
 
     def _resolve_attribute(self, name):
+        if self.resource_id is None:
+            return
         share = self._request_share()
         return six.text_type(getattr(share, name))
 
