@@ -20,18 +20,28 @@ from heat.engine import constraints
 from heat.engine import properties
 from heat.engine.resources.openstack.neutron import neutron
 from heat.engine import support
+from heat.engine import translation
 
 
 class Subnet(neutron.NeutronResource):
+    """A resource for managing Neutron subnets.
+
+    A subnet represents an IP address block that can be used for assigning IP
+    addresses to virtual instances. Each subnet must have a CIDR and must be
+    associated with a network. IPs can be either selected from the whole subnet
+    CIDR, or from "allocation pools" that can be specified by the user.
+    """
 
     PROPERTIES = (
-        NETWORK_ID, NETWORK, CIDR, VALUE_SPECS, NAME, IP_VERSION,
-        DNS_NAMESERVERS, GATEWAY_IP, ENABLE_DHCP, ALLOCATION_POOLS,
-        TENANT_ID, HOST_ROUTES, IPV6_RA_MODE, IPV6_ADDRESS_MODE,
+        NETWORK_ID, NETWORK, SUBNETPOOL, PREFIXLEN, CIDR,
+        VALUE_SPECS, NAME, IP_VERSION, DNS_NAMESERVERS, GATEWAY_IP,
+        ENABLE_DHCP, ALLOCATION_POOLS, TENANT_ID, HOST_ROUTES,
+        IPV6_RA_MODE, IPV6_ADDRESS_MODE,
     ) = (
-        'network_id', 'network', 'cidr', 'value_specs', 'name', 'ip_version',
-        'dns_nameservers', 'gateway_ip', 'enable_dhcp', 'allocation_pools',
-        'tenant_id', 'host_routes', 'ipv6_ra_mode', 'ipv6_address_mode',
+        'network_id', 'network', 'subnetpool', 'prefixlen', 'cidr',
+        'value_specs', 'name', 'ip_version', 'dns_nameservers', 'gateway_ip',
+        'enable_dhcp', 'allocation_pools', 'tenant_id', 'host_routes',
+        'ipv6_ra_mode', 'ipv6_address_mode',
     )
 
     _ALLOCATION_POOL_KEYS = (
@@ -87,17 +97,30 @@ class Subnet(neutron.NeutronResource):
             ],
             support_status=support.SupportStatus(version='2014.2')
         ),
+        SUBNETPOOL: properties.Schema(
+            properties.Schema.STRING,
+            _('The name or ID of the subnet pool.'),
+            constraints=[
+                constraints.CustomConstraint('neutron.subnetpool')
+            ],
+            support_status=support.SupportStatus(version='6.0.0'),
+        ),
+        PREFIXLEN: properties.Schema(
+            properties.Schema.INTEGER,
+            _('Prefix length for subnet allocation from subnet pool.'),
+            constraints=[constraints.Range(min=0)],
+            support_status=support.SupportStatus(version='6.0.0'),
+        ),
         CIDR: properties.Schema(
             properties.Schema.STRING,
             _('The CIDR.'),
-            required=True,
             constraints=[
                 constraints.CustomConstraint('net_cidr')
             ]
         ),
         VALUE_SPECS: properties.Schema(
             properties.Schema.MAP,
-            _('Extra parameters to include in the creation request.'),
+            _('Extra parameters to include in the request.'),
             default={},
             update_allowed=True
         ),
@@ -122,9 +145,12 @@ class Subnet(neutron.NeutronResource):
         ),
         GATEWAY_IP: properties.Schema(
             properties.Schema.STRING,
-            _('The gateway IP address. Set to any of [ null | ~ | "" ] to '
-              'create the subnet without a gateway. If omitted, the first IP '
-              'address within the subnet is assigned to the gateway.'),
+            _('The gateway IP address. Set to any of [ null | ~ | "" ] '
+              'to create/update a subnet without a gateway. '
+              'If omitted when creation, neutron will assign the first '
+              'free IP address within the subnet to the gateway '
+              'automatically. If remove this from template when update, '
+              'the old gateway IP address will be detached.'),
             update_allowed=True
         ),
         ENABLE_DHCP: properties.Schema(
@@ -141,6 +167,7 @@ class Subnet(neutron.NeutronResource):
                 schema={
                     ALLOCATION_POOL_START: properties.Schema(
                         properties.Schema.STRING,
+                        _('Start address for the allocation pool.'),
                         required=True,
                         constraints=[
                             constraints.CustomConstraint('ip_addr')
@@ -148,6 +175,7 @@ class Subnet(neutron.NeutronResource):
                     ),
                     ALLOCATION_POOL_END: properties.Schema(
                         properties.Schema.STRING,
+                        _('End address for the allocation pool.'),
                         required=True,
                         constraints=[
                             constraints.CustomConstraint('ip_addr')
@@ -159,20 +187,26 @@ class Subnet(neutron.NeutronResource):
         ),
         TENANT_ID: properties.Schema(
             properties.Schema.STRING,
-            _('The ID of the tenant who owns the network. Only administrative'
-              ' users can specify a tenant ID other than their own.')
+            _('The ID of the tenant who owns the network. Only administrative '
+              'users can specify a tenant ID other than their own.')
         ),
         HOST_ROUTES: properties.Schema(
             properties.Schema.LIST,
+            _('A list of host route dictionaries for the subnet.'),
             schema=properties.Schema(
                 properties.Schema.MAP,
                 schema={
                     ROUTE_DESTINATION: properties.Schema(
                         properties.Schema.STRING,
-                        required=True
+                        _('The destination for static route.'),
+                        required=True,
+                        constraints=[
+                            constraints.CustomConstraint('net_cidr')
+                        ]
                     ),
                     ROUTE_NEXTHOP: properties.Schema(
                         properties.Schema.STRING,
+                        _('The next hop for the destination.'),
                         required=True,
                         constraints=[
                             constraints.CustomConstraint('ip_addr')
@@ -184,8 +218,7 @@ class Subnet(neutron.NeutronResource):
         ),
         IPV6_RA_MODE: properties.Schema(
             properties.Schema.STRING,
-            _('IPv6 RA (Router Advertisement) mode.'
-              ' dhcpv6-stateful, dhcpv6-stateless, or slaac.'),
+            _('IPv6 RA (Router Advertisement) mode.'),
             constraints=[
                 constraints.AllowedValues([DHCPV6_STATEFUL, DHCPV6_STATELESS,
                                            SLAAC]),
@@ -194,8 +227,7 @@ class Subnet(neutron.NeutronResource):
         ),
         IPV6_ADDRESS_MODE: properties.Schema(
             properties.Schema.STRING,
-            _('IPv6 address mode.'
-              ' dhcpv6-stateful, dhcpv6-stateless, or slaac.'),
+            _('IPv6 address mode.'),
             constraints=[
                 constraints.AllowedValues([DHCPV6_STATEFUL, DHCPV6_STATELESS,
                                            SLAAC]),
@@ -249,10 +281,27 @@ class Subnet(neutron.NeutronResource):
 
     def translation_rules(self, props):
         return [
-            properties.TranslationRule(props,
-                                       properties.TranslationRule.REPLACE,
-                                       [self.NETWORK],
-                                       value_path=[self.NETWORK_ID])
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.REPLACE,
+                [self.NETWORK],
+                value_path=[self.NETWORK_ID]),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.NETWORK],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='network'
+            ),
+            translation.TranslationRule(
+                props,
+                translation.TranslationRule.RESOLVE,
+                [self.SUBNETPOOL],
+                client_plugin=self.client_plugin(),
+                finder='find_resourceid_by_name_or_id',
+                entity='subnetpool'
+            )
         ]
 
     @classmethod
@@ -269,8 +318,21 @@ class Subnet(neutron.NeutronResource):
 
     def validate(self):
         super(Subnet, self).validate()
+        subnetpool = self.properties[self.SUBNETPOOL]
+        prefixlen = self.properties[self.PREFIXLEN]
+        cidr = self.properties[self.CIDR]
+        if subnetpool is not None and cidr:
+            raise exception.ResourcePropertyConflict(self.SUBNETPOOL,
+                                                     self.CIDR)
+        if subnetpool is None and not cidr:
+            raise exception.PropertyUnspecifiedError(self.SUBNETPOOL,
+                                                     self.CIDR)
+        if prefixlen and cidr:
+            raise exception.ResourcePropertyConflict(self.PREFIXLEN,
+                                                     self.CIDR)
         ra_mode = self.properties[self.IPV6_RA_MODE]
         address_mode = self.properties[self.IPV6_ADDRESS_MODE]
+
         if (self.properties[self.IP_VERSION] == 4) and (
                 ra_mode or address_mode):
             msg = _('ipv6_ra_mode and ipv6_address_mode are not supported '
@@ -292,9 +354,10 @@ class Subnet(neutron.NeutronResource):
         props = self.prepare_properties(
             self.properties,
             self.physical_resource_name())
-        self.client_plugin().resolve_network(props, self.NETWORK, 'network_id')
+        props['network_id'] = props.pop(self.NETWORK)
+        if self.SUBNETPOOL in props and props[self.SUBNETPOOL]:
+            props['subnetpool_id'] = props.pop('subnetpool')
         self._null_gateway_ip(props)
-
         subnet = self.client().create_subnet({'subnet': props})['subnet']
         self.resource_id_set(subnet['id'])
 
@@ -310,12 +373,17 @@ class Subnet(neutron.NeutronResource):
         return self.client().show_subnet(self.resource_id)['subnet']
 
     def handle_update(self, json_snippet, tmpl_diff, prop_diff):
-        props = self.prepare_update_properties(json_snippet)
-        if (self.ALLOCATION_POOLS in prop_diff and
-                self.ALLOCATION_POOLS not in props):
-            props[self.ALLOCATION_POOLS] = []
-        self.client().update_subnet(
-            self.resource_id, {'subnet': props})
+        if prop_diff:
+            self.prepare_update_properties(prop_diff)
+            if (self.ALLOCATION_POOLS in prop_diff and
+                    prop_diff[self.ALLOCATION_POOLS] is None):
+                prop_diff[self.ALLOCATION_POOLS] = []
+
+            # If the new value is '', set to None
+            self._null_gateway_ip(prop_diff)
+
+            self.client().update_subnet(
+                self.resource_id, {'subnet': prop_diff})
 
 
 def resource_mapping():
