@@ -13,8 +13,8 @@
 
 import mock
 
+from heat.common import exception
 from heat.common import template_format
-from heat.engine import resources
 from heat.engine.resources.openstack.mistral import cron_trigger
 from heat.engine import scheduler
 from heat.tests import common
@@ -43,14 +43,14 @@ class FakeCronTrigger(object):
         self.remaining_executions = 3
         self._data = {'trigger': 'info'}
 
+    def to_dict(self):
+        return self._data
+
 
 class MistralCronTriggerTest(common.HeatTestCase):
 
     def setUp(self):
         super(MistralCronTriggerTest, self).setUp()
-        resources.initialise()
-        utils.setup_dummy_db()
-        self.ctx = utils.dummy_context()
 
         t = template_format.parse(stack_template)
         self.stack = utils.parse_stack(t)
@@ -63,24 +63,26 @@ class MistralCronTriggerTest(common.HeatTestCase):
 
     def _create_resource(self, name, snippet, stack):
         ct = cron_trigger.CronTrigger(name, snippet, stack)
+        mock_get_workflow = mock.Mock(return_value='get_first_glance_image')
+        ct.client_plugin().get_workflow_by_identifier = mock_get_workflow
         self.client.cron_triggers.create.return_value = FakeCronTrigger(
             'my_cron_trigger')
         self.client.cron_triggers.get.return_value = FakeCronTrigger(
             'my_cron_trigger')
         scheduler.TaskRunner(ct.create)()
-        args = self.client.cron_triggers.create.call_args[1]
-        self.assertEqual('* * 0 * *', args['pattern'])
-        self.assertEqual('get_first_glance_image', args['workflow_name'])
-        self.assertEqual({}, args['workflow_input'])
-        self.assertEqual('2015-04-08 06:20', args['first_time'])
-        self.assertEqual(3, args['count'])
-        self.assertEqual('my_cron_trigger', ct.resource_id)
         return ct
 
     def test_create(self):
         ct = self._create_resource('trigger', self.rsrc_defn, self.stack)
         expected_state = (ct.CREATE, ct.COMPLETE)
         self.assertEqual(expected_state, ct.state)
+        args, kwargs = self.client.cron_triggers.create.call_args
+        self.assertEqual('* * 0 * *', kwargs['pattern'])
+        self.assertEqual('get_first_glance_image', args[1])
+        self.assertEqual({}, kwargs['workflow_input'])
+        self.assertEqual('2015-04-08 06:20', kwargs['first_time'])
+        self.assertEqual(3, kwargs['count'])
+        self.assertEqual('my_cron_trigger', ct.resource_id)
 
     def test_attributes(self):
         ct = self._create_resource('trigger', self.rsrc_defn, self.stack)
@@ -88,3 +90,34 @@ class MistralCronTriggerTest(common.HeatTestCase):
                          ct.FnGetAtt('next_execution_time'))
         self.assertEqual(3, ct.FnGetAtt('remaining_executions'))
         self.assertEqual({'trigger': 'info'}, ct.FnGetAtt('show'))
+
+    def test_validate_fail(self):
+        t = template_format.parse(stack_template)
+        del t['resources']['cron_trigger']['properties']['first_time']
+        del t['resources']['cron_trigger']['properties']['pattern']
+        stack = utils.parse_stack(t)
+        resource_defns = stack.t.resource_definitions(stack)
+        self.rsrc_defn = resource_defns['cron_trigger']
+        ct = self._create_resource('trigger', self.rsrc_defn, self.stack)
+        msg = ("At least one of the following properties must be specified: "
+               "pattern, first_time")
+        self.assertRaisesRegexp(exception.PropertyUnspecifiedError, msg,
+                                ct.validate)
+
+    def test_validate_ok_without_first_time(self):
+        t = template_format.parse(stack_template)
+        del t['resources']['cron_trigger']['properties']['first_time']
+        stack = utils.parse_stack(t)
+        resource_defns = stack.t.resource_definitions(stack)
+        self.rsrc_defn = resource_defns['cron_trigger']
+        ct = self._create_resource('trigger', self.rsrc_defn, self.stack)
+        ct.validate()
+
+    def test_validate_ok_without_pattern(self):
+        t = template_format.parse(stack_template)
+        del t['resources']['cron_trigger']['properties']['pattern']
+        stack = utils.parse_stack(t)
+        resource_defns = stack.t.resource_definitions(stack)
+        self.rsrc_defn = resource_defns['cron_trigger']
+        ct = self._create_resource('trigger', self.rsrc_defn, self.stack)
+        ct.validate()
